# -*- coding: utf-8 -*-
from __future__ import absolute_import, unicode_literals

import base64
import json
import aiohttp
import tempfile
from datetime import datetime
from base64 import encodebytes

import diskcache
from dateutil.tz import tzutc
from dateutil.parser import parse
import boto.utils
from urllib.parse import urlparse, parse_qs, quote
from boto import provider
from boto.connection import HTTPRequest
from boto.auth import HmacAuthV3HTTPHandler

from .constants import APP_KEY, HOST, USER_AGENT, BASE_URI


class ZuluHmacAuthV3HTTPHandler(HmacAuthV3HTTPHandler):

    def sign_string(self, string_to_sign):
        new_hmac = self._get_hmac()
        new_hmac.update(string_to_sign)
        return encodebytes(new_hmac.digest()).decode('utf-8').strip()

    def headers_to_sign(self, http_request):
        headers_to_sign = {'Host': self.host}
        for name, value in http_request.headers.items():
            lname = name.lower()
            if lname.startswith('x-amz'):
                headers_to_sign[name] = value
        return headers_to_sign

    def canonical_query_string(self, http_request):
        if http_request.method == 'POST':
            return ''
        qs_parts = []
        for param in sorted(http_request.params):
            value = boto.utils.get_utf8_value(http_request.params[param])
            param_ = quote(param, safe='-_.~')
            value_ = quote(value, safe='-_.~')
            qs_parts.append('{0}={1}'.format(param_, value_))
        return '&'.join(qs_parts)

    def string_to_sign(self, http_request):
        headers_to_sign = self.headers_to_sign(http_request)
        canonical_qs = self.canonical_query_string(http_request)
        canonical_headers = self.canonical_headers(headers_to_sign)
        string_to_sign = '\n'.join((
            http_request.method,
            http_request.path,
            canonical_qs,
            canonical_headers,
            '',
            http_request.body
        ))
        return string_to_sign, headers_to_sign


async def _get_credentials():
    url = '{0}/authentication/credentials/temporary/ios82'.format(BASE_URI)
    async with aiohttp.ClientSession() as session:
        async with session.post(url,
                                json={'appKey': APP_KEY},
                                headers={'User-Agent': USER_AGENT}) as res:
            res.raise_for_status()
            data = await res.json(encoding='utf-8')
    return data['resource']


class Auth(object):

    SOON_EXPIRES_SECONDS = 60
    _CREDS_STORAGE_KEY = 'aioimdb-credentials'

    def __init__(self, creds=None):
        self._cachedir = tempfile.gettempdir()

    def _get_creds(self):
        with diskcache.Cache(directory=self._cachedir) as cache:
            return cache.get(self._CREDS_STORAGE_KEY)

    def _set_creds(self, creds):
        with diskcache.Cache(directory=self._cachedir) as cache:
            cache[self._CREDS_STORAGE_KEY] = creds
        return creds

    def clear_cached_credentials(self):
        with diskcache.Cache(directory=self._cachedir) as cache:
            cache.delete(self._CREDS_STORAGE_KEY)

    def _creds_soon_expiring(self):
        creds = self._get_creds()
        if not creds:
            return creds, True
        expires_at = parse(creds['expirationTimeStamp'])
        now = datetime.now(tzutc())
        if now < expires_at:
            time_diff = expires_at - now
            if time_diff.total_seconds() < self.SOON_EXPIRES_SECONDS:
                # creds will soon expire, so renew them
                return creds, True
            return creds, False
        else:
            return creds, True

    async def get_auth_headers(self, url_path):
        creds, soon_expires = self._creds_soon_expiring()
        if soon_expires:
            creds = self._set_creds(creds=await _get_credentials())

        handler = ZuluHmacAuthV3HTTPHandler(
            host=HOST,
            config={},
            provider=provider.Provider(
                name='aws',
                access_key=creds['accessKeyId'],
                secret_key=creds['secretAccessKey'],
                security_token=creds['sessionToken'],
            )
        )
        parsed_url = urlparse(url_path)
        params = {
            key: val[0] for key, val in parse_qs(parsed_url.query).items()
        }
        request = HTTPRequest(
            method='GET', protocol='https', host=HOST,
            port=443, path=parsed_url.path, auth_path=None, params=params,
            headers={}, body=''
        )
        handler.add_auth(req=request)
        headers = request.headers
        headers['User-Agent'] = USER_AGENT
        return headers



# coding: utf-8
from __future__ import absolute_import, unicode_literals

from operator import itemgetter

import pytest

from aioimdb import Imdb


@pytest.fixture(scope='function')
async def client():
    async with Imdb(locale='en_US') as client:
        yield client
    client.clear_cached_credentials()


@pytest.mark.asyncio
async def test_get_title_plot_synopsis(client):
    expected_keys = [
        '@type', 'id', 'plotSynopses', 'title', 'titleType', 'year'
    ]

    resource = await client.get_title_plot_synopsis('tt0111161')

    assert sorted(resource.keys()) == sorted(expected_keys)


@pytest.mark.asyncio
async def test_get_title_plot(client):
    expected_keys = ['@type', 'outline', 'summaries', 'totalSummaries']

    resource = await client.get_title_plot('tt0111161')

    assert sorted(resource.keys()) == sorted(expected_keys)


@pytest.mark.asyncio
async def test_get_title_user_reviews(client):
    expected_keys = [
        '@type', 'base', 'paginationKey', 'reviews', 'totalReviews'
    ]

    resource = await client.get_title_user_reviews('tt0111161')

    assert sorted(resource.keys()) == sorted(expected_keys)


@pytest.mark.asyncio
async def test_get_title_metacritic_reviews(client):
    expected_keys = [
        '@type', 'id', 'metaScore', 'metacriticUrl', 'reviewCount',
        'userRatingCount', 'userScore', 'reviews', 'title'
    ]

    resource = await client.get_title_metacritic_reviews('tt0111161')

    assert sorted(resource.keys()) == sorted(expected_keys)


@pytest.mark.asyncio
async def test_title_reviews_non_existant_title(client):
    with pytest.raises(LookupError):
        await client.get_title_user_reviews('tt9999999')


@pytest.mark.asyncio
async def test_title_exists(client):
    result = await client.title_exists('tt2322441')
    assert True is result


@pytest.mark.asyncio
async def test_title_exists_non_existant_title(client):
    result = await client.title_exists('tt0000000')
    assert False is result


@pytest.mark.asyncio
async def test_search_for_title_searching_title(client):
    results = await client.search_for_title('Shawshank redemption')
    expected_top_results = [
        {
            'imdb_id': 'tt0111161',
            'title': 'The Shawshank Redemption',
            'year': '1994',
            'type': 'feature',
        },
        {
            'imdb_id': 'tt5443386',
            'title': 'The Shawshank Redemption: Behind the Scenes',
            'year': '2004',
            'type': 'video',
        },
    ]
    assert len(results) > 0
    assert expected_top_results == results[:2]


@pytest.mark.asyncio
@pytest.mark.parametrize('query', [
    'Mission: Impossible',
    'Honey, I Shrunk the Kids',
    '4.3.2.1. (2010)',
    '500 Days of Summer (2009)',
    '$9.99 (2008)',
    'Goonies 1986',
    '[REC] (2007)',
    '[REC]² (2009)',
    '[REC]³ Genesis (2012)',
    '¡Three Amigos! (1986)',
    '(Untitled) (2009)',
])
async def test_search_for_title_input_with_special_chars(query, client):
    results = await client.search_for_title(query)
    assert len(results) > 0


@pytest.mark.asyncio
async def test_search_for_name(client):
    results = await client.search_for_name('Andrew Lloyd Webber')

    assert len(results) > 0
    expected_results = [
        {'name': 'Andrew Lloyd Webber', 'imdb_id': 'nm0515908'},
    ]
    assert (sorted(expected_results, key=itemgetter('imdb_id')) ==
            sorted(results, key=itemgetter('imdb_id')))


@pytest.mark.asyncio
async def test_search_for_title_no_results(client):
    results = await client.search_for_title('898582da396c93d5589e0')
    assert [] == results


@pytest.mark.asyncio
async def test_get_popular_titles(client):
    expected_keys = ['@type', 'id', 'ranks']

    resource = await client.get_popular_titles()

    assert sorted(resource.keys()) == sorted(expected_keys)
    assert len(resource['ranks']) == 100


@pytest.mark.asyncio
async def test_get_popular_shows(client):
    expected_keys = ['@type', 'id', 'ranks']

    resource = await client.get_popular_shows()

    assert sorted(resource.keys()) == sorted(expected_keys)


@pytest.mark.asyncio
async def test_get_popular_movies(client):
    expected_keys = ['@type', 'id', 'ranks']

    resource = await client.get_popular_shows()

    assert sorted(resource.keys()) == sorted(expected_keys)


@pytest.mark.asyncio
async def test_get_name(client):
    expected_keys = [
        '@type', 'base', 'id', 'jobs', 'knownFor', 'quotes',
        'trivia'
    ]

    resource = await client.get_name('nm0000151')

    assert sorted(resource.keys()) == sorted(expected_keys)


@pytest.mark.asyncio
async def test_get_name_filmography(client):
    expected_keys = ['@type', 'filmography']

    resource = await client.get_name_filmography('nm0000151')

    assert sorted(resource.keys()) == sorted(expected_keys)


@pytest.mark.asyncio
async def test_get_title(client):
    imdb_id = 'tt0111161'
    expected_keys = [
        '@type', 'base', 'filmingLocations', 'metacriticScore', 'plot',
        'ratings', 'similarities', 'soundtrack'
    ]

    resource = await client.get_title(imdb_id)

    assert sorted(resource.keys()) == sorted(expected_keys)


@pytest.mark.asyncio
async def test_get_title_genres(client):
    imdb_id = 'tt0111161'
    expected_keys = ['@type', 'genres', 'id', 'title', 'titleType', 'year']

    resource = await client.get_title_genres(imdb_id)

    assert sorted(resource.keys()) == sorted(expected_keys)


@pytest.mark.asyncio
async def test_get_title_similarities(client):
    imdb_id = 'tt0111161'
    expected_keys = ['@type', 'base', 'id', 'similarities']

    resource = await client.get_title_similarities(imdb_id)

    assert sorted(resource.keys()) == sorted(expected_keys)


@pytest.mark.asyncio
async def test_get_title_awards(client):
    imdb_id = 'tt0111161'
    expected_keys = ['@type', 'awards', 'id']

    resource = await client.get_title_awards(imdb_id)

    assert sorted(resource.keys()) == sorted(expected_keys)


@pytest.mark.asyncio
async def test_get_title_releases(client):
    imdb_id = 'tt0111161'
    expected_keys = ['@type', 'releases', 'id', 'title', 'titleType', 'year']

    resource = await client.get_title_releases(imdb_id)

    assert sorted(resource.keys()) == sorted(expected_keys)


@pytest.mark.asyncio
async def test_get_title_versions(client):
    imdb_id = 'tt0111161'
    expected_keys = [
        '@type', 'alternateTitles', 'alternateVersions', 'colorations',
        'defaultTitle', 'silent', 'spokenLanguages', 'originalTitle',
        'origins', 'runningTimes', 'id', 'title', 'titleType', 'year'
    ]

    resource = await client.get_title_versions(imdb_id)

    assert sorted(resource.keys()) == sorted(expected_keys)


@pytest.mark.asyncio
async def test_get_title_ratings(client):
    imdb_id = 'tt0111161'
    expected_keys = [
        '@type', 'id', 'title', 'titleType', 'year', 'bottomRank',
        'canRate', 'rating', 'ratingCount', 'topRank'
    ]

    resource = await client.get_title_ratings(imdb_id)

    assert sorted(resource.keys()) == sorted(expected_keys)


@pytest.mark.asyncio
async def test_get_title_quotes(client):
    imdb_id = 'tt0111161'
    expected_keys = ['@type', 'quotes', 'id', 'title', 'titleType', 'year']

    resource = await client.get_title_quotes(imdb_id)

    assert sorted(resource.keys()) == sorted(expected_keys)


@pytest.mark.asyncio
async def test_get_title_connections(client):
    imdb_id = 'tt0111161'
    expected_keys = ['@type', 'base', 'connections']

    resource = await client.get_title_connections(imdb_id)

    assert sorted(resource.keys()) == sorted(expected_keys)


@pytest.mark.asyncio
async def test_get_title_credits(client):
    imdb_id = 'tt0111161'
    expected_keys = ['@type', 'base', 'creditsSummary', 'id', 'credits']

    resource = await client.get_title_credits(imdb_id)

    assert sorted(resource.keys()) == sorted(expected_keys)


@pytest.mark.asyncio
async def test_get_title_credits_with_redirection_result(client):
    redir_imdb_id = 'tt0000021'

    with pytest.raises(LookupError):
        await client.get_title_credits(redir_imdb_id)


@pytest.mark.asyncio
async def test_get_title_redirection_result(client):
    redir_imdb_id = 'tt0000021'

    with pytest.raises(LookupError):
        await client.get_title(redir_imdb_id)


@pytest.mark.asyncio
async def test_get_title_excludes_episodes(client):
    episode_imdb_id = 'tt3181538'
    assert await client.get_title(episode_imdb_id) is not None

    with pytest.raises(LookupError) as exc:
        async with Imdb(exclude_episodes=True) as imdb:
            await imdb.get_title(episode_imdb_id)

    exc.match(r'Title not found. Title was an episode.+')


@pytest.mark.asyncio
async def test_get_title_episodes(client):
    tv_show_imdb_id = 'tt0303461'
    expected_keys = ['@type', 'base', 'id', 'seasons']

    resource = await client.get_title_episodes(tv_show_imdb_id)

    assert sorted(resource.keys()) == sorted(expected_keys)


@pytest.mark.asyncio
async def test_get_title_episodes_raises_when_exclude_episodes_enabled():
    async with Imdb(exclude_episodes=True) as client:
        with pytest.raises(ValueError):
            await client.get_title_episodes('tt0303461')


@pytest.mark.asyncio
async def test_get_title_episodes_raises_imdb_id_is_not_that_of_a_tv_show(
        client):
    non_show_imdb_id = 'tt0468569'
    with pytest.raises(LookupError):
        await client.get_title_episodes(non_show_imdb_id)


@pytest.mark.asyncio
async def test_get_name_images(client):
    expected_keys = ['@type', 'images', 'totalImageCount']

    resource = await client.get_name_images('nm0000032')

    assert sorted(resource.keys()) == sorted(expected_keys)


@pytest.mark.asyncio
async def test_get_name_videos(client):
    expected_keys = [
        '@type', 'akas', 'id', 'image', 'legacyNameText', 'name', 'size',
        'videoCounts', 'videos'
    ]

    resource = await client.get_name_videos('nm0000032')

    assert sorted(resource.keys()) == sorted(expected_keys)


@pytest.mark.asyncio
async def test_get_title_images(client):
    expected_keys = ['@type', 'images', 'totalImageCount']

    resource = await client.get_title_images('tt0111161')

    assert sorted(resource.keys()) == sorted(expected_keys)


@pytest.mark.asyncio
async def test_get_title_videos(client):
    expected_keys = [
        '@type', 'id', 'image', 'size', 'title', 'titleType', 'videoCounts',
        'videos', 'year'
    ]

    resource = await client.get_title_videos('tt0111161')

    assert sorted(resource.keys()) == sorted(expected_keys)


@pytest.mark.asyncio
async def test_get_title_raises_not_found(client):
    with pytest.raises(LookupError):
        await client.get_title('tt9999999')


@pytest.mark.asyncio
@pytest.mark.parametrize('imdb_id, exp_valid', [
    ('tt1234567', True),
    ('nm1234567', True),
    ('x', False),
    (1234567, False),
    (None, False),
])
async def test_validate_imdb_id(imdb_id, exp_valid, client):

    if exp_valid:
        # no raise
        client.validate_imdb_id(imdb_id)
    else:
        with pytest.raises(ValueError):
            client.validate_imdb_id(imdb_id)



# -*- coding: utf-8 -*-
from __future__ import absolute_import, unicode_literals

import hashlib

HOST = 'api.imdbws.com'
BASE_URI = 'https://{0}'.format(HOST)
SEARCH_BASE_URI = 'https://v2.sg.media-imdb.com'
USER_AGENT = 'IMDb/8.3.1 (iPhone9,4; iOS 11.2.1)'
APP_KEY = '76a6cc20-6073-4290-8a2c-951b4580ae4a'



import pytest
from freezegun import freeze_time

from aioimdb.auth import Auth


@pytest.fixture
def auth():
    auth_ = Auth()
    auth_._set_creds({'expirationTimeStamp': '2018-01-12T06:23:05Z'})
    yield auth_
    auth_.clear_cached_credentials()


@pytest.mark.parametrize('current_datetime, exp_expired', [
    ('2018-01-12T06:23:05Z', True),  # matching expiry time
    ('2018-01-12T06:24:05Z', True),  # 1 min after expiry time
    ('2018-01-12T06:23:04Z', True),  # 1 sec before, must be 60+ before
    ('2018-01-12T06:22:06Z', True),  # 59 sec before, must be 60+ before
    ('2018-01-12T06:22:05Z', False),  # 60 sec before, must be 60+ before
    ('2018-01-11T06:23:05Z', False),  # 1 day before expiry time
    ('2017-11-11T06:23:05Z', False),  # a few weeks before expiry time
])
def test_creds_soon_expiring(auth, current_datetime, exp_expired):
    with freeze_time(current_datetime):
        assert auth._creds_soon_expiring()[1] is exp_expired



# -*- coding: utf-8 -*-
from __future__ import absolute_import, unicode_literals

import re
import json
import tempfile
import time
import random
import logging
import datetime
import warnings
from string import Template
from http import HTTPStatus
from urllib.parse import urlencode, quote, quote_plus, unquote, urlparse
import aiohttp

from .constants import BASE_URI, HOST, SEARCH_BASE_URI
from .auth import Auth
from .exceptions import ImdbAPIError

logger = logging.getLogger(__name__)


ENDPOINTS = {
    'name': Template('/name/${imdb_id}/fulldetails'),
    'name_filmography': Template('/name/${imdb_id}/filmography'),
    'name_images': Template('/name/${imdb_id}/images'),
    'name_videos': Template('/name/${imdb_id}/videos'),
    'title_credits': Template('/title/${imdb_id}/fullcredits'),
    'title_quotes': Template('/title/${imdb_id}/quotes'),
    'title_ratings': Template('/title/${imdb_id}/ratings'),
    'title_genres': Template('/title/${imdb_id}/genres'),
    'title_similarities': Template('/title/${imdb_id}/similarities'),
    'title_awards': Template('/title/${imdb_id}/awards'),
    'title_connections': Template('/title/${imdb_id}/connections'),
    'title_releases': Template('/title/${imdb_id}/releases'),
    'title_versions': Template('/title/${imdb_id}/versions'),
    'title_plot': Template('/title/${imdb_id}/plot'),
    'title_plot_synopsis': Template('/title/${imdb_id}/plotsynopsis'),
    'title_images': Template('/title/${imdb_id}/images'),
    'title_videos': Template('/title/${imdb_id}/videos'),
    'title_user_reviews': Template('/title/${imdb_id}/userreviews'),
    'title_metacritic_reviews': Template('/title/${imdb_id}/metacritic'),
    # '': Template('/title/${imdb_id}'),
}


class Imdb(Auth):

    def __init__(self, locale=None, exclude_episodes=False, session=None):
        self.locale = locale or 'en_US'
        self.exclude_episodes = exclude_episodes
        self.session = session or aiohttp.ClientSession()
        self._cachedir = tempfile.gettempdir()

    async def __aenter__(self):
        return self

    async def __aexit__(self, etype, evalue, etb):
        if self.session is not None:
            await self.session.close()

    async def _fetcher(self, name, imdb_id, endpoints=ENDPOINTS):
        if name not in endpoints:
            raise NotImplementedError(f'The endpoint {name} is not defined.')

        if name.startswith('title'):
            await self._redirection_title_check(imdb_id)

        logger.info(f'getting {name} {imdb_id}')
        self.validate_imdb_id(imdb_id)
        url = endpoints[name].substitute(imdb_id=imdb_id)
        return await self._get_resource(url)

    def __getattr__(self, method_name):
        if not method_name.startswith('get_'):
            return super().__getattr__(method_name)

        name = method_name[4:]
        if name not in ENDPOINTS:
            return super().__getattr__(method_name)
        return lambda *args, **kwargs: self._fetcher(name, *args, **kwargs)

    async def get_title(self, imdb_id):
        logger.info(f'getting title {imdb_id}')
        self.validate_imdb_id(imdb_id)
        await self._redirection_title_check(imdb_id)
        try:
            resource = await self._get_resource(f'/title/{imdb_id}/auxiliary')
        except LookupError:
            self._title_not_found()

        if (
            self.exclude_episodes is True and
            resource['base']['titleType'] == 'tvEpisode'
        ):
            raise LookupError(
                'Title not found. Title was an episode and '
                '"exclude_episodes" is set to true'
            )
        return resource

    async def title_exists(self, imdb_id):
        self.validate_imdb_id(imdb_id)
        page_url = f'http://www.imdb.com/title/{imdb_id}/'

        async with self.session.head(page_url) as response:
            if response.status == HTTPStatus.OK:
                return True
            elif response.status == HTTPStatus.NOT_FOUND:
                return False
            elif response.status == HTTPStatus.MOVED_PERMANENTLY:
                # redirection result
                return False
            else:
                response.raise_for_status()

    async def search_for_name(self, name):
        logger.info(f'searching for name {name}')
        name = re.sub(r'\W+', '_', name).strip('_')
        query = quote(name)
        first_alphanum_char = self._query_first_alpha_num(name)
        url = (
            '{0}/suggests/{1}/{2}.json'.format(SEARCH_BASE_URI,
                                               first_alphanum_char, query)
        )
        search_results = await self._get(url=url, query=query)
        results = []
        for result in search_results.get('d', ()):
            if not result['id'].startswith('nm'):
                # ignore non-person results
                continue
            result_item = {
                'name': result['l'],
                'imdb_id': result['id'],
            }
            results.append(result_item)
        return results

    async def search_for_title(self, title):
        logger.info(f'searching for title {title}')
        title = re.sub(r'\W+', '_', title).strip('_')
        query = quote(title)
        first_alphanum_char = self._query_first_alpha_num(title)
        url = (
            '{0}/suggests/{1}/{2}.json'.format(SEARCH_BASE_URI,
                                               first_alphanum_char, query)
        )
        search_results = await self._get(url=url, query=query)
        results = []
        for result in search_results.get('d', ()):
            result_item = {
                'title': result['l'],
                'year': str(result['y']) if result.get('y') else None,
                'imdb_id': result['id'],
                'type': result.get('q'),
            }
            results.append(result_item)
        return results

    async def get_popular_titles(self):
        return await self._get_resource('/chart/titlemeter')

    async def get_popular_shows(self):
        return await self._get_resource('/chart/tvmeter')

    async def get_popular_movies(self):
        return await self._get_resource('/chart/moviemeter')

    async def get_title_episodes(self, imdb_id):
        logger.info(f'getting title {imdb_id} episodes')
        self.validate_imdb_id(imdb_id)
        if self.exclude_episodes:
            raise ValueError('exclude_episodes is current set to true')
        return await self._get_resource(f'/title/{imdb_id}/episodes')

    @staticmethod
    def _cache_response(file_path, resp):
        with open(file_path, 'w+') as f:
            json.dump(resp, f)

    def _parse_dirty_json(self, data, query=None):
        if query is None:
            match_json_within_dirty_json = r'imdb\$.+\({1}(.+)\){1}'
        else:
            query_match = ''.join(
                char if char.isalnum() else f'[{char}]'
                for char in unquote(query)
            )
            query_match = query_match.replace('[ ]', '.+')
            match_json_within_dirty_json = (
                r'imdb\${}\((.+)\)'.format(query_match)
            )
        data_clean = re.match(
            match_json_within_dirty_json, data, re.IGNORECASE
        ).groups()[0]
        return json.loads(data_clean)

    @staticmethod
    def validate_imdb_id(imdb_id):
        match_id = r'[a-zA-Z]{2}[0-9]{7}'
        try:
            re.match(match_id, imdb_id, re.IGNORECASE).group()
        except (AttributeError, TypeError):
            raise ValueError('invalid imdb id')

    @staticmethod
    def _is_redirection_result(response):
        """
        Return True if response is that of a redirection else False
        Redirection results have no information of use.
        """
        imdb_id = response['data'].get('tconst')
        if (
            imdb_id and
            imdb_id != response['data'].get('news', {}).get('channel')
        ):
            return True
        return False

    async def _get_resource(self, path):
        url = f'{BASE_URI}{path}'
        data = await self._get(url=url)
        return data['resource']

    async def _get(self, url, query=None):
        path = urlparse(url).path
        headers = {'Accept-Language': self.locale}
        headers.update(await self.get_auth_headers(path))

        async with self.session.get(url, headers=headers) as resp:
            if not resp.status == HTTPStatus.OK:
                if resp.status == HTTPStatus.NOT_FOUND:
                    raise LookupError(f'Resource {path} not found')
                else:
                    msg = f'{resp.status} {resp.text}'
                    raise ImdbAPIError(msg)
            resp_data = await resp.text(encoding='utf-8')
            try:
                resp_dict = json.loads(resp_data)
            except ValueError:
                resp_dict = self._parse_dirty_json(
                    data=resp_data, query=query
                )

            if resp_dict.get('error'):
                return None

        return resp_dict

    async def _redirection_title_check(self, imdb_id):
        if await self.is_redirection_title(imdb_id):
            self._title_not_found(msg=f'{imdb_id} is a redirection imdb id')

    async def is_redirection_title(self, imdb_id):
        self.validate_imdb_id(imdb_id)
        page_url = f'http://www.imdb.com/title/{imdb_id}/'
        async with self.session.head(page_url) as response:
            if response.status == HTTPStatus.MOVED_PERMANENTLY:
                return True
            else:
                return False

    def _query_first_alpha_num(self, query):
        for char in query.lower():
            if char.isalnum():
                return char
        raise ValueError(
            'invalid query, does not contain any alphanumeric characters'
        )

    def _title_not_found(self, msg=''):
        if msg:
            msg = f' {msg}'
        raise LookupError(f'Title not found.{msg}')



#!/usr/bin/env python
# -*- coding: utf-8 -*
import codecs
import os
import re
from setuptools import setup


here = os.path.abspath(os.path.dirname(__file__))


def read(*parts):
    with codecs.open(os.path.join(here, *parts), 'r') as fp:
        return fp.read()


def find_version(*file_paths):
    version_file = read(*file_paths)
    version_match = re.search(r"^__version__ = ['\"]([^'\"]*)['\"]",
                              version_file, re.M)
    if version_match:
        return version_match.group(1)
    raise RuntimeError('Unable to find version string.')


with open('requirements.txt') as f:
    install_requires = f.read().splitlines()

with open('README.md') as f:
    long_description = f.read()

setup(
    name='aioimdb',
    version=find_version('aioimdb', '__init__.py'),
    packages=['aioimdb', ],
    include_package_data=True,
    zip_safe=False,
    description=(
        'Python asyncio IMDB client using the IMDB json web service made '
        'available for their iOS app.'
    ),
    author='Francesco Pierfederici',
    author_email='fpierfed@megabeets.com',
    license='Creative Commons Attribution-Noncommercial-Share Alike license',
    long_description=long_description,
    long_description_content_type='text/markdown',
    url='https://github.com/fpierfed/aioimdb',
    install_requires=install_requires,
    python_requires='>=3.6',
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Intended Audience :: Developers',
        'Operating System :: OS Independent',
        'License :: OSI Approved :: Apache Software License',
        'Framework :: AsyncIO',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3.6',
        'Topic :: Internet :: WWW/HTTP',
    ],
)
